package org.kie.examples.jta;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.UserTransaction;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.TargetsContainer;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ArchivePaths;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.resolver.api.maven.Maven;
import org.junit.Assert;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.runtime.KieSession;

/**
 *
 */
public class TransactionTest extends AbstractContainerTest {

    private static final String KIE_BASE_NAME = "sampleKBase";
    private static final String DRL_RESOURCE_NAME = "sample.drl";

    @Deployment
    @TargetsContainer(MANAGED_CONTAINER)
    public static Archive<?> deployManaged() {
        WebArchive war = ShrinkWrap.create(WebArchive.class, "test.war");
        war.addAsLibraries(Maven.resolver().loadPomFromFile("pom.xml")
                .importRuntimeAndTestDependencies()
                .resolve().withTransitivity().asFile());

        war.addPackages(true, TransactionTest.class.getPackage());
        war.addAsResource(TransactionTest.class.getPackage(), DRL_RESOURCE_NAME);

        war.addAsWebInfResource(TransactionTest.class.getResource("/META-INF/kmodule.xml"),
                ArchivePaths.create("classes/META-INF/kmodule.xml"));
        war.addAsWebInfResource(TransactionTest.class.getResource("/META-INF/pom.properties"),
                ArchivePaths.create("classes/META-INF/maven/group/artifact/pom.properties"));
        war.addAsResource(TransactionTest.class.getResource("/server-persistence.xml"),
                ArchivePaths.create("META-INF/persistence.xml"));

        return war;
    }

    @Test
    public void testCommit() throws Exception {
        KieSession ksession = createJPASession(getKnowledgeBase());

        //final Object fact = new Person("user-transaction");
        final Object fact = "user-transaction";

        UserTransaction utx = getTransaction();
        utx.begin();

        ksession = reloadSession(ksession);
        ksession.insert(fact);
        ksession.fireAllRules();

        utx.commit();

        ksession = reloadSession(ksession);

        Assert.assertEquals(1, ksession.getObjects().size());
        Assert.assertEquals(fact, ksession.getObjects().iterator().next());
    }

    @Test
    public void testRollback() throws Exception {
        KieSession ksession = createJPASession(getKnowledgeBase());

        //final Object fact = new Person("user-transaction");
        final Object fact = "user-transaction";

        UserTransaction utx = getTransaction();
        utx.begin();

        ksession = reloadSession(ksession);
        ksession.insert(fact);
        ksession.fireAllRules();

        utx.rollback();

        ksession = reloadSession(ksession);

        Assert.assertEquals(0, ksession.getObjects().size());
    }

    private KieBase getKnowledgeBase() {
        return getServices().getKieClasspathContainer().getKieBase(KIE_BASE_NAME);
    }

    private static UserTransaction getTransaction() {
        try {
            return (UserTransaction) InitialContext.doLookup("java:comp/UserTransaction");
        } catch (NamingException ex) {
            throw new RuntimeException("Could not lookup transaction", ex);
        }
    }

}
